var str; // variable for character cleansing
var sso, sso3d; // No comments		 
var dsSwymUrl,d3SwymUrl,dsSwymPassportUrl,d3SwymPassportUrl; // Variable to store used defined url

	// Set of functions to convert inappropriate special characters in JSON data and fix them.	
	function convertXML2Char (str) {
	// converts XML or HTML text to characters by removing all character entities and ncrs
	// str: string, the sequence to be converted

	// remove various escaped forms
		str = convertHexNCR2Char(str);
		str = convertDecNCR2Char(str);
		str = convertEntities2Char(str);
	
	return str;
	}
	
	function convertHexNCR2Char ( str ) { 
	// converts a string containing &#x...; escapes to a string of characters
	// str: string, the input
	
	// convert up to 6 digit escapes to characters
			str = str.replace(/&#x([A-Fa-f0-9]{1,6});/g, 
				function(matchstr, parens) {
					return hex2char(parens);
				}
			); 
	return str;
	}

	function convertDecNCR2Char ( str ) { 
	// converts a string containing &#...; escapes to a string of characters
	// str: string, the input
	
	// convert up to 6 digit escapes to characters
			str = str.replace(/&#([0-9]{1,7});/g, 
				function(matchstr, parens) {
					return dec2char(parens);
				}
			);
	return str;
	}

	function convertEntities2Char ( str ) { 
	// converts a string containing HTML/XML character entities to a string of characters
	// str: string, the input
	
		str = str.replace(/&([A-Za-z0-9]+);/g, 
			function(matchstr, parens) { //alert(parens);
				if (parens in entities) { //alert(entities[parens]);
					return entities[parens];
				} 
				else { return matchstr; }						
			}
		);
		return str;
	}

	function hex2char ( hex ) {
	// converts a single hex number to a character
	// note that no checking is performed to ensure that this is just a hex number, eg. no spaces etc
	// hex: string, the hex codepoint to be converted
	var result = '';
	var n = parseInt(hex, 16);
    if (n <= 0xFFFF) { result += String.fromCharCode(n); } 
	else if (n <= 0x10FFFF) {
		n -= 0x10000
		result += String.fromCharCode(0xD800 | (n >> 10)) + String.fromCharCode(0xDC00 | (n & 0x3FF));
    	} 
	else { result += 'hex2Char error: Code point out of range: '+dec2hex(n); }
	return result;
	}
	
	function dec2char ( n ) {
	// converts a single string representing a decimal number to a character
	// note that no checking is performed to ensure that this is just a hex number, eg. no spaces etc
	// dec: string, the dec codepoint to be converted
	var result = '';
    if (n <= 0xFFFF) { result += String.fromCharCode(n); } 
	else if (n <= 0x10FFFF) {
		n -= 0x10000
		result += String.fromCharCode(0xD800 | (n >> 10)) + String.fromCharCode(0xDC00 | (n & 0x3FF));
    	} 
	else { result += 'dec2char error: Code point out of range: '+dec2hex(n); }
	return result;
	}		

	
	// Get URL from local storage (for both 3DSWYM and DSSWYM), which was set by user in the extension option.	
	function getURLFromUser(){
		
		browser.storage.local.set('dsSwymUrl', function (val){      // Get DSSWYM URL stored in local storage
			dsSwymUrl= val.dsSwymUrl;
			//console.log("Debug: URL recieved from sync data for DSSWYM: "+ dsSwymUrl); // Debug Check point
		});		
		
		browser.storage.local.set('d3SwymUrl', function (val){	 // Get 3DSWYM URL stored in local storage
			d3SwymUrl= val.d3SwymUrl;
			//console.log("Debug: URL recieved from sync data for 3DSWYM: "+ d3SwymUrl);	// Debug Check point
		});		 
		
	}

	function determineDSSwymPassportServer(){
		$.ajax({
			url:'https://'+dsSwymUrl+'/api/passport/login',			
			type: "GET",		
			contentType: 'text/html; charset=UTF-8', 
			crossDomain: true
		}).done(function(data){
				// Nothing here
		}).fail(function(xhr, status, error){			
			console.log("DSSWYM XHR error on passportURL: "+xhr);			// User Message
			console.log("DSSWYM Status error on passportURL: "+status);	// User Message
			console.log("DSSWYM Error on passportURL: "+error);     // User Message
		});						
	}
	
	function determine3DSwymPassportServer(){
		$.ajax({
			url:'https://'+d3SwymUrl+'/api/passport/login',			
			type: "GET",		
			contentType: 'text/html; charset=UTF-8', 
			crossDomain: true
		}).done(function(data){
			// Nothing here
		}).fail(function(xhr, status, error){			
			console.log("3DSWYM XHR error on passportURL: "+xhr);			// User Message
			console.log("3DSWYM Status error on passportURL: "+status);	// User Message
			console.log("3DSWYM Error on passportURL: "+error);     // User Message
		});				
	}
	
	// No comments
	function getSSO(){
		$.ajax({
			url:'https://'+dsSwymUrl+'/api/index/tk',			
			type: "GET",				
			dataType: 'json',
			contentType: 'application/json; charset=utf-8', 
			crossDomain: true
		}).done(function(data){
			sso = data.result.ServerToken;		
			chrome.storage.sync.set({'SSO': sso}, function () {
				//console.log("Debug: DSSWYM SSO stored in storage: "+sso);    // Debug Check point
			});						  
		}).fail(function(xhr, status, error){			
			console.log("DSSWYM xhr upon SSO request: "+xhr);			// User Message
			console.log("DSSWYM Status upon SSO request: "+status);	// User Message
			console.log("DSSWYM Error upon SSO request: "+error);     // User Message
			if(error == "Unauthorized" || error == "Authorization Required"){				// This log would tell how often token fails to make service calls or validity of token can be measured, reliability of token
				console.log("Authorization required, call to function authenticateDSSwym");	// Message to user
				authenticateDSSwym();
			}
		});				
	}

	function getSSO3d(){
		$.ajax({
			url:'https://'+d3SwymUrl+'/api/index/tk',			
			type: "GET",			
			dataType: 'json',
			contentType: 'application/json; charset=utf-8', 
			crossDomain: true
		}).done(function(data, status, xhr){
			//console.log("Debug: SSO recieved from 3DSWYM: "+xhr.getResponseHeader('X-SWYM')); // Debug Check point
			sso3d = xhr.getResponseHeader('X-SWYM');		
			chrome.storage.sync.set({'SSO3d': sso3d}, function () {
				//console.log("Debug: 3DSWYM SSO stored in storage: "+sso3d);    // Debug Check point
			});						
		}).fail(function(xhr, status, error){			
			console.log("3DSWYM xhr upon SSO request: "+xhr);			//User Message
			console.log("3DSWYM Status upon SSO request: "+status);     //User Message
			console.log("3DSWYM Error upon SSO request: "+error);       //User Message
			if(error == "Unauthorized" || error == "Authorization Required"){				// This log would tell how often token fails to make service calls or validity of token can be measured, reliability of token
				console.log("Authorization required, call to function authenticate3dSwym");	// Message to user
				authenticate3dSwym();
			}			
		});				
	}	

	
	// Set of Function to ask user to login, so that 1 week of access to data can be assured	
	function authenticateDSSwym(){ 		
		// Notify user that login ticket is expired and ask him to click the notification to login 		
		// Authenticate notification pops up every 30 minutes
		if(dsSwymPassportUrl != "" && dsSwymPassportUrl != undefined ){
			$.ajax({
				url: dsSwymPassportUrl+'/login',			
				type: 'GET',	
				data:{'service':'https://'+dsSwymUrl+'/api/passport/login'},		   
				contentType: 'application/x-www-form-urlencoded; charset=UTF-8', 
				crossDomain: true
			}).done(function(data,testStatus, xhr){
				//console.log("Debug: 3dPassport authentication is successfull in DSSWYM"); //Debug check point
				//console.log("Debug: On Success data DSSwYm: "+data);
				//console.log("Debug: On Success testStaus DSSwYm: "+testStatus);
				//console.log("Debug: On Success xhr DSSwYm: "+xhr);
				//console.log("Debug: DSSWYM Respone header Swym-Auth-Ok: "+xhr.getResponseHeader('Swym-Auth-Ok'));
				if(xhr.status === 200 && xhr.getResponseHeader('Swym-Auth-Ok') == "1"){
					//console.log("Debug: Presence of ticket is noticed in DSSWYM");
				}else if(xhr.status === 200 && !(xhr.getResponseHeader('Swym-Auth-Ok'))){
					//console.log("Debug: /browser cookie expired or cleared for DSSWYM/ Initiated notificaiton to the user, to authenticate.");
					chrome.notifications.create("dsswym",
					{ 	type: "basic",
						title: "Authorization required",
						message: 'Browser cookie is expired, time to login!',
						iconUrl: 'authorize.jpg',
						//eventTime: 300000,
						requireInteraction: true,
						buttons: [{ title: "Authorize now", iconUrl: 'key.png'},
								{ title: "Disable the notification", iconUrl: 'disable.png'}]						
					});	
				}				
			}).fail(function(xhr, status, error){			
				console.log("DSSWYM Passport xhr of Login request: "+xhr);			//User message
				console.log("DSSWYM Passport Status of Login request: "+status);	//User message
				console.log("DSSWYM Passport Error of Login request: "+error);		//User message
			});	
		}else{
			determineDSSwymPassportServer();	
			//console.log("Debug: Function call to determine the 3DPassport URL for DSSWYM"); //Debug check point
		}		
	}	

	function authenticate3dSwym(){ // Similar to above function
		if(d3SwymPassportUrl != "" && d3SwymPassportUrl != undefined ){
			$.ajax({
				url: d3SwymPassportUrl+'/cas/login',			
				type: 'GET',	
				data:{'service':'https://'+d3SwymUrl+'/api/passport/login'}, //,gateway:true},  
				contentType: 'application/x-www-form-urlencoded', 
				crossDomain: true
			}).done(function(data,testStatus, xhr){
				//console.log("Debug: Passport authentication is successfull in 3DSWYM"); //Debug check point
				//console.log("Debug: On Success data 3DSwYm: "+data);
				//console.log("Debug: On Success testStaus 3DSwYm: "+testStatus);
				//console.log("Debug: On Success xhr 3DSwYm: "+xhr);
				//console.log("Debug: 3DSWYM Respone header Swym-Auth-Ok: "+xhr.getResponseHeader('Swym-Auth-Ok'));
				if(xhr.status === 200 && xhr.getResponseHeader('Swym-Auth-Ok') == "1"){
					//console.log("Debug: Presence of ticket is noticed in 3DSWYM");
				}else if(xhr.status === 200 && !(xhr.getResponseHeader('Swym-Auth-Ok'))){
					//console.log("Debug: /browser cookie expired or cleared/ Initiated notificaiton to the user, to authenticate 3DSWYM."); //Debug check point
					chrome.notifications.create("3dswym",
					{ 	type: "basic",
						title: "Authorization required",
						message: 'Browser cookie is expired, time for login!',
						iconUrl: 'authorize.jpg',
						//eventTime: 300000,
						requireInteraction: true,
						buttons: [{ title: "Authorize now", iconUrl: 'key.png'},
								{ title: "Disable the notification", iconUrl: 'disable.png'}]						
					});	
				}				
			}).fail(function(xhr, status, error){			
				console.log(" 3DSWYM Passport xhr of Login request: "+xhr);				//User message
				console.log(" 3DSWYM Passport Status of Login request: "+status);		//User message
				console.log(" 3DSWYM Passport Error of Login request: "+error);			//User message
			});	
		}else{
			determine3DSwymPassportServer();	
			//console.log("Debug: Function call to determine the Passport URL for 3DSWYM"); //Debug check point
		}
	}

	
	//Based on user preference stored in local storage for enable/disable DSSWYM notification, this function is called.
	//This function calls dsPost & dsIquestion function every 30 seconds.
	setInterval(function(){
		chrome.storage.sync.get('dsSwymNotification', function (val) {
			//console.log("Debug: Do i've the permission to fetch notification for User: "+ val.dsSwymNotification);	// Debug Check point		
			if(val.dsSwymNotification =="yes"){
				if((dsSwymUrl != "") && (dsSwymUrl != undefined)){	// Ensure SWYM URL is defined by the user.
					//dsPost();
					//dsIquestion();
					dsIdea();
				}else{
					getURLFromUser();								// Function call to get url from user i.e. from local storage
				}
			}					
		});			
	}, 10000);  // In testing Mode
	//}, 50000);  // In testing Mode, when supplied to testers
	//}, 900000);	// In deployment mode.
	
	// set of functions to make web service calls to get bulk information and send individual piece of data to respective notificaiton function.
	// It has history tracker, which tracks the record of already displayed notificaiton upto 2 last displayed notification.
	function dsPost(){
		var dsPostTitle, lastLink, author, authorId, community, extract,thumbnail_id; // Variable to store the information for notification
		var historyCheck, decoy, initialDecoyCounter; // decoy variables
		var stringify; // temporary varaible to clense dirty JSON data
		var timer; // To introduce dealy in execution
		//console.log("Initial value of historycheck: "+historyCheck);
	
		if(dsSwymUrl == undefined || dsSwymUrl== ""){ // Call function to get URL stored in local storage
			console.log('User has to define DSSWYM URL');  // Message to user
			getURLFromUser();						
		}	
		
		chrome.storage.sync.get('lastDsPost', function (val) { // Get the last notified link from local storage and assign to decoy variable for comparison down the line
			historyCheck = val.lastDsPost;
			//console.log("Debug: Initializing the value from sync data to historyCheck to compare upcoming post with notified history : "+historyCheck);
		});

		chrome.storage.sync.get('decoyPost', function (val) { // Get the last "but one" notified link from local storage and assign to decoy variable for comparison down the line
			decoy = val.decoyPost;
			//console.log("Debug: Initializing the value from sync data to decoy for decoy action : "+decoy);
		});
		
		chrome.storage.sync.get('SSO', function (val) { // No comments.
			sso = val.SSO;			
		});
	
		timer ="start";	// variable to ensure this loop is completed before the next statement & ajax request are executed.
		var i = 0;
		do {			// In order to reduce data weight only 2 latest items are fetched for comparison.	
			i += 1;
			$.ajax({
				url:'https://'+dsSwymUrl+'/api/post/list',			
				type: "GET",			
				data:{"params":{"community_id":null,"limit":2}},		    
				headers: {'X-Requested-With': 'XMLHttpRequest','X-DS-SWYM-CSRFTOKEN': sso },
				dataType: 'json',
				contentType: 'application/json; charset=utf-8', 
				crossDomain: true
			}).done(function(data){
				//console.log("Debug: Economy loop executed in DSSWYM Post");
				stringify = JSON.stringify(data);
				extract = JSON.parse(stringify);
				//console.log(extract); // Debug Check point
				stringify=null; 
				
				if(historyCheck == undefined || historyCheck == ""){			// This block executes only the first time extension is used or reset triggered.
				//console.log("Debug: historyCheck is undefined, value of history check: "+historyCheck);
				// This piece of code is used for testing purpose to enable quick debugging of JSON package extractions.
			/*	//  Required information are title, author, user_id, community_name, thumbnail_id, postlink
					dsPostTitle = extract.result[0].title;
					console.log("dsPostTitle: "+dsPostTitle);
					author = extract.result[0].author.last_name + " " + extract.result[0].author.first_name;
					console.log("author: "+author);
					authorId = extract.result[0].author.login;
					console.log("authorId: "+authorId);
					community = extract.result[0].communityTitle;
					console.log("community: "+community); 
					thumbnail_id = extract.result[0]['this->getFirstMediaId'];
					console.log("thumbnail_id : "+thumbnail_id);
					lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/post:"+extract.result[0].id;
					console.log("lastLink: "+lastLink);
			*/
					dsPostTitle = extract.result[0].title; 			 // extract post title
					dsPostTitle = convertXML2Char(dsPostTitle);		 // Inappropriate special characters fixing function call.
					author = extract.result[0].author.last_name + " " + extract.result[0].author.first_name; // extract author name
					author = convertXML2Char(author);  				 // Inappropriate special characters fixing function call.
					authorId = extract.result[0].author.login; 		 // extract author login i.e. trigram
					community = extract.result[0].communityTitle;	 // extract post title
					community = convertXML2Char(community);	 		 // Inappropriate special characters fixing function call.			
					thumbnail_id = extract.result[0]['this->getFirstMediaId']; // extract first image ID of the Post
					lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/post:"+extract.result[0].id; //extract information to construct URL to the post					
					
					initialDecoyCounter ="https://"+dsSwymUrl+"/#community:"+extract.result[1].community_id+"/post:"+extract.result[1].id; // decoy URL
					
					chrome.storage.sync.set({'lastDsPost': lastLink}, function () { 								// Storing last notification URL in local storage
						//console.log("Debug: Last post notification is stored for DSSWYM Post: "+lastLink); 				// Debug Check point
					});				
					chrome.storage.sync.set({'decoyPost': initialDecoyCounter}, function () {						// Storing decoy URL in local storage
						//console.log("Debug: Last but one Post notification for DSSWYM is stored: "+initialDecoyCounter); // Debug Check point
					});
						
					dspostnotifier(dsPostTitle,lastLink, author,authorId,community,thumbnail_id);  					// Call to notification function with parameters of information
					extract = null;
					data=null;		
					timer = 'finished';					
					//console.log("Debug: Break command processed by POST function");					
					return;
					console.log("~~~~~~~~Return not honoured under post function, This should not happen~~~~~~~~");						
				}
				
				lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/post:"+extract.result[0].id; 
				
				data  = null;
				extract = null;	
				timer = 'finished';				
			}).fail(function(xhr, status, error){
				timer = 'errorHit';
				console.log("DSPost xhr log: "+xhr); 			// Message to user
				console.log("DSPost Status log: "+status);		// Message to user
				console.log("DSPost Error log: "+error);		// Message to user
				if(error == "Unauthorized" || error == "Authorization Required"){				// This log would tell how often token fails to make service calls or validity of token can be measured, reliability of token
					console.log("Authorization required, call to function authenticateDSSwym");	// Message to user
					authenticateDSSwym();
				} 
				if(error == "Forbidden"){														// This log would fetch the fresh token in case of forbidden error // Need to investigate this type of error cause
					console.log("Call to SSO made from DSWYM POST Function");					// Message to user
					getSSO();				
				}	
			});	
		} while (i < 1);	
		i=null;
		
		var delayBolt = setInterval(function () {		// This bolt will wait till previous do loop completes execution.
			if(timer == "finished"){					// This is variable which is hard coded to detemine the completion of previous do loop.
				clearInterval(delayBolt);				// This will clear the looping process within this function.
		
				if(historyCheck == lastLink || historyCheck == undefined){  	// Mostly function will hit this point, when there is no update to be notified.
					//console.log("Debug: found match at starting, there is no update yet in DSSWYM Post"); // Debug Check point
					return; 
				}else {															// Last 10 information is fetched and compared.
					//console.log("Debug: Updates happened, started processing under DSWSYM Post function"); // Debug check point
					$.ajax({
						url:'https://'+dsSwymUrl+'/api/post/list',			
						type: "GET",			
						data:{"params":{"community_id":null,"limit":10}},		    
						headers: {'X-Requested-With': 'XMLHttpRequest','X-DS-SWYM-CSRFTOKEN': sso },
						dataType: 'json',
						contentType: 'application/json; charset=utf-8', 
						crossDomain: true
					}).done(function(data){
						//console.log("Debug: Success in getting the data under DSSWYM");
						stringify = JSON.stringify(data);
						extract = JSON.parse(stringify);
						//console.log(extract); // Debug Check point
						stringify=null; 
							
							for (var r=0; r<10;r++){ 

								dsPostTitle = extract.result[r].title; // Refer do while block comments.
								dsPostTitle = convertXML2Char(dsPostTitle);
								author = extract.result[r].author.last_name + " " + extract.result[r].author.first_name;
								author = convertXML2Char(author);
								authorId = extract.result[r].author.login;
								community = extract.result[r].communityTitle;
								community = convertXML2Char(community);					
								thumbnail_id = extract.result[r]['this->getFirstMediaId'];
								lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[r].community_id+"/post:"+extract.result[r].id;						
								
								 //console.log("Debug: For the value of r: "+r+" value of initialdsp and decoy are:"+historyCheck+" and "+decoy);  // Debug Check point 
												
								if(historyCheck == lastLink){  
									//console.log("Debug: found match at: "+r+ " There is no update yet"); // Debug Check point
									data = null;
									extract = null;
									return;
								}
								
								if((decoy == lastLink)&& (r!=9)){
									//console.log("Debug: decoy at action under DSSWYM POST"); 				 								 		// Debug Check point
									chrome.storage.sync.set({'lastDsPost': lastLink}, function () {
										//console.log("Debug: Last Post notification for DSSWYM is updated to: "+lastLink); 		// Debug Check point
									});
									initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[r+1].community_id+"/post:"+extract.result[r+1].id;
									chrome.storage.sync.set({'decoyPost': initialDecoyCounter}, function () {
										//console.log("Debug: Last Post notification for DSSWYM is updated to: "+initialDecoyCounter);// Debug Check point
									});
									data = null;
									extract = null;
									return; 
								}
								
								dspostnotifier(dsPostTitle,lastLink,author,authorId,community,thumbnail_id);			// Call to notification function with parameters of information
								
								if(r==0){  // Core logic of this code is to store the latest update e.g. first avaialble post from JSON to be compared with next set of updates in JSON. Hence r==0 is evaluated.
									chrome.storage.sync.set({'lastDsPost': lastLink}, function () {
										//console.log("Debug: After Notification update to lastdspost "+lastLink);			// Debug Check point
									});
									initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[r+1].community_id+"/post:"+extract.result[r+1].id;
									chrome.storage.sync.set({'decoyPost': initialDecoyCounter}, function () {
										//console.log("Debug: After Notification update to decoy: "+initialDecoyCounter);	// Debug Check point
									});
								}
							} 
							data = null;
							extract = null;
						
					}).fail(function(xhr, status, error){
						console.log("DSPOST xhr log: "+xhr); 			// Message to user
						console.log("DSPOST Status log: "+status);		// Message to user
						console.log("DSPOST Error log: "+error);		// Message to user
						if(error == "Unauthorized" || error == "Authorization Required"){				// This log would tell how often token fails to make service calls or validity of token can be measured, reliability of token
							console.log("Authorization required, call to function authenticateDSSwym");	// Message to user
							authenticateDSSwym();
						}
						if(error == "Forbidden"){														// This log would fetch the fresh token in case of forbidden error // Need to investigate this type of error cause
							console.log("Call to SSO made from DSWYM POST Function");					// Message to user
							getSSO();				
						}			
					});
				}
			}else if(timer == "errorHit"){
				clearInterval(delayBolt);
				return;
			}
		}, 100);		
	}	
		
	function dsIquestion(){
		var titleDsQuestion, lastLink, extract, author, authorId, community, thumbnail_id; 
		var initialDecoyCounter, historyCheck, decoy; 
		var stringify;
		var timer;
		
		if(dsSwymUrl == undefined || dsSwymUrl== ""){
			console.log('User has to define DSSWYM URL'); // Message to user
			getURLFromUser();						
		}			
		
		chrome.storage.sync.get('decoyQuestion', function (val) {
			decoy = val.decoyQuestion;
			//console.log("Debug: Initializing the value from sync data to decoy for decoy action: "+decoy);
		}); 			
		
		chrome.storage.sync.get('lastDsQuestion', function (val) {
			historyCheck = val.lastDsQuestion;
			//console.log("Debug: Initializing the value from sync to historyCheck for question update: "+historyCheck);
		}); 
	
		timer = 'start';
		var i = 0;
		do {
			i += 1;
			$.ajax({
				url:'https://'+dsSwymUrl+'/api/question/list',			
				type: "GET",
				data:{"params":{"community_id":null,"limit":2,"published":1}},
				headers: {'X-Requested-With': 'XMLHttpRequest','X-DS-SWYM-CSRFTOKEN': sso},
				dataType: 'json',
				contentType: 'application/json; charset=utf-8',
				crossDomain: true
			}).done(function(data){
				//console.log("Debug: Economy loop executed in DSSWYM IQuestion");
				stringify = JSON.stringify(data);
				extract = JSON.parse(stringify);
				//console.log(extract); // Debug Check point
				stringify=null; 

				if(historyCheck == undefined || historyCheck == ""){ // This block applicable only on first time notification happens or when user reset the local storage.				
							
					// This piece of code is used for testing purpose to enable quick debugging of JSON package extractions.
				/*	//  Required information are title, author, user_id, community_name, thumbnail_id, postlink
					titleDsQuestion = extract.result[0].title;
					console.log("titleDsQuestion: "+titleDsQuestion);
					author = extract.result[0].author.last_name + " " + extract.result[0].author.first_name;
					console.log("author: "+author);
					authorId = extract.result[0].author.login;
					console.log("authorId: "+authorId);
					community = extract.result[0].community_title;
					console.log("community: "+community); 
					thumbnail_id = extract.result[0].first_media_id;
					console.log("thumbnail_id : "+thumbnail_id);
					lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/iquestion:"+extract.result[0].qna_id;
					console.log("lastLink: "+lastLink); 
				*/			
					titleDsQuestion = extract.result[0].title;
					titleDsQuestion = convertXML2Char(titleDsQuestion);
					author = extract.result[0].author.last_name + " " + extract.result[0].author.first_name;
					author = convertXML2Char(author);
					authorId = extract.result[0].author.login;
					community = extract.result[0].community_title;
					community = convertXML2Char(community);
					thumbnail_id = extract.result[0].first_media_id;
					lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/iquestion:"+extract.result[0].qna_id;			
									
					initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[1].community_id+"/iquestion:"+extract.result[1].qna_id;
					
					chrome.storage.sync.set({'lastDsQuestion': lastLink}, function () {
						//console.log("Debug: Last IQuestion notification for DSSWYM is stored"+lastLink);
					});		
					chrome.storage.sync.set({'decoyQuestion': initialDecoyCounter}, function () {
						//console.log("Debug: Last but one IQuestion notification for DSSWYM is stored"+initialDecoyCounter);
					});
					
					if(titleDsQuestion !=""){
						dsIQuestionnotifier(titleDsQuestion,lastLink,author,authorId,community,thumbnail_id); // Call to notification function with parameters of information
					}	
					data = null;
					extract=null;
					timer = 'finished';					
					//console.log("Debug: Break command processed by Question Function");
					return;
					console.log("~~~~~~~~~Break not honoured under Question Function. This should not happen~~~~~~~~");					
				}
				lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/iquestion:"+extract.result[0].qna_id;			
				
				data  = null;
				extract = null;	
				timer = 'finished'; 
			}).fail(function(xhr, status, error){
				timer = 'errorHit';
				console.log("DSIQuestion xhr log: "+xhr); 			// Message to user
				console.log("DSIQuestion Status log: "+status);		// Message to user
				console.log("DSIQuestion Error log: "+error);		// Message to user
				if(error =="Unauthorized"){
					console.log("Authorization Required, a message from DSSWYM IQuestion Function"); // User message, post function handles this error message.
				}				
			});	
			data  = null;
			extract = null;		
		} while (i < 1);	
		i=null;

		var delayBolt = setInterval(function () {
			if(timer == "finished"){
				clearInterval(delayBolt);
				
					if(historyCheck == lastLink || historyCheck == undefined){  
						//console.log("Debug: Found match at starting, there is no update yet in DSSWYM IQuestion"); // Debug Check point
						return; 
					}else{	
						//console.log("Debug: Updates happened, started processing under DSWSYM IQuestion function"); // Debug check point		
						$.ajax({
							url:'https://'+dsSwymUrl+'/api/question/list',			
							type: "GET",
							data:{"params":{"community_id":null,"limit":10,"published":1}},
							headers: {'X-Requested-With': 'XMLHttpRequest','X-DS-SWYM-CSRFTOKEN': sso},
							dataType: 'json',
							contentType: 'application/json; charset=utf-8',
							crossDomain: true
						}).done(function(data){
							stringify = JSON.stringify(data);
							extract = JSON.parse(stringify);
							//console.log(extract); // Debug Check point
							stringify=null;
				
								for (var k=0; k<10;k++){ 

									titleDsQuestion = extract.result[k].title;
									titleDsQuestion = convertXML2Char(titleDsQuestion);
									author = extract.result[k].author.last_name + " " + extract.result[k].author.first_name;
									author = convertXML2Char(author);
									authorId = extract.result[k].author.login;
									community = extract.result[k].community_title;
									community = convertXML2Char(community);
									thumbnail_id = extract.result[k].first_media_id;
									lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[k].community_id+"/iquestion:"+extract.result[k].qna_id;
									
									//console.log("Debug: For the value of k: "+k+" value of initialdsq and decoy are:"+historyCheck+" and "+decoy);
									
									if(historyCheck == lastLink){  
										//console.log("Debug: Found match at: "+k+" There is no update yet for DSIQuestion");							
										data = null;
										extract=null;
										return; 				
									} 
									
									if(decoy == lastLink){
										//console.log("Debug: decoy at action in DSIQuestion");													// Debug Check point
										chrome.storage.sync.set({'lastDsQuestion': lastLink}, function () {
											//console.log("Debug: Last IQuestion notification for DSSWYM is updated to: "+lastLink);				// Debug Check point
										});
										initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[k+1].community_id+"/iquestion:"+extract.result[k+1].qna_id;
										chrome.storage.sync.set({'decoyQuestion': initialDecoyCounter}, function () {
											//console.log("Debug: Last IQuestion notification for decoy is updated to : "+initialDecoyCounter);	// Debug Check point
										});
										data = null;
										extract=null;
										return; 
									}
									
									if(titleDsQuestion != ""){
										dsIQuestionnotifier(titleDsQuestion,lastLink,author,authorId,community,thumbnail_id); 			// Call to notification function with parameters of information
										if(k==0){
											chrome.storage.sync.set({'lastDsQuestion': lastLink}, function () {
												//console.log("Debug: After Notification update to lastdsq "+lastLink);						// Debug Check point
											});
											initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[k+1].community_id+"/iquestion:"+extract.result[k+1].qna_id;
											chrome.storage.sync.set({'decoyQuestion': initialDecoyCounter}, function () {
												//console.log("Debug: After Notification update to decoy "+initialDecoyCounter);				// Debug Check point
											});
										}					
									}
								}
								data = null;
								extract=null;
							
						}).fail(function(xhr, status, error){
							console.log("DSIquestion xhr: "+xhr);
							console.log("DSIquestion status: "+status);
							console.log("DSIquestion Error: "+error);
							if(error =="Unauthorized"){ // This part is handled by POST function.
								console.log("Authorization Required, a message from DSSWYM IQuestion Function");  // User message
							}
						}); 
					}
				}else if(timer == "errorHit"){
				clearInterval(delayBolt);
				return;
			}
		}, 100);
	}

	function dsIdea(){
		var dsIdeaTitle, lastLink, author, authorId, community, extract,thumbnail_id; // Variable to store the information for notification
		var historyCheck, decoy, initialDecoyCounter; // decoy variables
		var stringify; // temporary varaible to clense dirty JSON data
		var timer; // To introduce dealy in execution
		//console.log("Initial value of historycheck: "+historyCheck);
	
		if(dsSwymUrl == undefined || dsSwymUrl== ""){ // Call function to get URL stored in local storage
			console.log('User has to define DSSWYM URL');  // Message to user
			getURLFromUser();						
		}	
		
		chrome.storage.sync.get('lastDsIdea', function (val) { // Get the last notified link from local storage and assign to decoy variable for comparison down the line
			historyCheck = val.lastDsIdea;
			//console.log("Debug: Initializing the value from sync data to historyCheck to compare upcoming Idea with notified history : "+historyCheck);
		});

		chrome.storage.sync.get('decoyIdea', function (val) { // Get the last "but one" notified link from local storage and assign to decoy variable for comparison down the line
			decoy = val.decoyIdea;
			//console.log("Debug: Initializing the value from sync data to decoy for decoy action : "+decoy);
		});
		
		chrome.storage.sync.get('SSO', function (val) { // No comments.
			sso = val.SSO;			
		});
	
		timer ="start";	// variable to ensure this loop is completed before the next statement & ajax request are executed.
		var i = 0;
		do {			// In order to reduce data weight only 2 latest items are fetched for comparison.	
			i += 1;
			$.ajax({
				url:'https://'+dsSwymUrl+'/api/idea/list',    //api/feed/whatsnew',		
				type: "GET",			
				data:{"params":{"community_id":null,"limit":2}},         //{"community_id":null,"limit":2}},		    
				headers: {'X-Requested-With': 'XMLHttpRequest','X-DS-SWYM-CSRFTOKEN': sso },
				dataType: 'json',
				contentType: 'application/json; charset=utf-8', 
				crossDomain: true
			}).done(function(data){
				console.log(data);
				//console.log("Debug: Economy loop executed in DSSWYM Idea");
				stringify = JSON.stringify(data);
				extract = JSON.parse(stringify);
				console.log(extract); // Debug Check point
				stringify=null; 
				
				if(historyCheck == undefined || historyCheck == ""){			// This block executes only the first time extension is used or reset triggered.
				//console.log("Debug: historyCheck is undefined, value of history check: "+historyCheck);
				// This piece of code is used for testing purpose to enable quick debugging of JSON package extractions.
				//  Required information are title, author, user_id, community_name, thumbnail_id, idealink
					dsIdeaTitle = extract.result[0].title;
					console.log("dsIdeaTitle: "+dsIdeaTitle);
					author = extract.result[0].author.last_name + " " + extract.result[0].author.first_name;
					console.log("author: "+author);
					authorId = extract.result[0].author.login;
					console.log("authorId: "+authorId);
					community = extract.result[0].community_title;
					console.log("community: "+community); 
					thumbnail_id = extract.result[0]['first_media_id'];
					console.log("thumbnail_id : "+thumbnail_id);
					lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/ideation:"+extract.result[0].id;
					console.log("lastLink: "+lastLink);
			
					dsIdeaTitle = extract.result[0].title; 			 // extract idea title
					dsIdeaTitle = convertXML2Char(dsIdeaTitle);		 // Inappropriate special characters fixing function call.
					author = extract.result[0].author.last_name + " " + extract.result[0].author.first_name; // extract author name
					author = convertXML2Char(author);  				 // Inappropriate special characters fixing function call.
					authorId = extract.result[0].author.login; 		 // extract author login i.e. trigram
					community = extract.result[0].community_title;	 // extract idea title
					community = convertXML2Char(community);	 		 // Inappropriate special characters fixing function call.			
					thumbnail_id = extract.result[0]['first_media_id']; // extract first image ID of the Idea
					lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/ideation:"+extract.result[0].id; //extract information to construct URL to the post					
					
					initialDecoyCounter ="https://"+dsSwymUrl+"/#community:"+extract.result[1].community_id+"/ideation:"+extract.result[1].id; // decoy URL
					
					chrome.storage.sync.set({'lastDsIdea': lastLink}, function () { 								// Storing last notification URL in local storage
						//console.log("Debug: Last idea notification is stored for DSSWYM Idea: "+lastLink); 				// Debug Check point
					});				
					chrome.storage.sync.set({'decoyIdea': initialDecoyCounter}, function () {						// Storing decoy URL in local storage
						//console.log("Debug: Last but one Idea notification for DSSWYM is stored: "+initialDecoyCounter); // Debug Check point
					});
						
					dsIdeanotifier(dsPostTitle,lastLink, author,authorId,community,thumbnail_id);  					// Call to notification function with parameters of information
					extract = null;
					data=null;		
					timer = 'finished';					
					//console.log("Debug: Break command processed by Idea function");					
					return;
					console.log("~~~~~~~~Return not honoured under Idea function, This should not happen~~~~~~~~");						
				}
				
				lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[0].community_id+"/ideation:"+extract.result[0].id; 
				
				data  = null;
				extract = null;	
				timer = 'finished';				
			}).fail(function(xhr, status, error){
				timer = 'errorHit';
				console.log("DSIdea xhr log: "+xhr); 			// Message to user
				console.log("DSIdea Status log: "+status);		// Message to user
				console.log("DSIdea Error log: "+error);		// Message to user
				if(error == "Unauthorized" || error == "Authorization Required"){				// This log would tell how often token fails to make service calls or validity of token can be measured, reliability of token
					console.log("Authorization required, call to function authenticateDSSwym");	// Message to user
					authenticateDSSwym();
				} 
				if(error == "Forbidden"){														// This log would fetch the fresh token in case of forbidden error // Need to investigate this type of error cause
					console.log("Call to SSO made from DSWYM Idea Function");					// Message to user
					getSSO();				
				}	
			});	
		} while (i < 1);	
		i=null;
		
	/*	var delayBolt = setInterval(function () {		// This bolt will wait till previous do loop completes execution.
			if(timer == "finished"){					// This is variable which is hard coded to detemine the completion of previous do loop.
				clearInterval(delayBolt);				// This will clear the looping process within this function.
		
				if(historyCheck == lastLink || historyCheck == undefined){  	// Mostly function will hit this point, when there is no update to be notified.
					//console.log("Debug: found match at starting, there is no update yet in DSSWYM Idea"); // Debug Check point
					return; 
				}else {															// Last 10 information is fetched and compared.
					//console.log("Debug: Updates happened, started processing under DSWSYM Idea function"); // Debug check point
					$.ajax({
						url:'https://'+dsSwymUrl+'/api/idea/list',			
						type: "GET",			
						data:{"params":{"community_id":null,"limit":10}},		    
						headers: {'X-Requested-With': 'XMLHttpRequest','X-DS-SWYM-CSRFTOKEN': sso },
						dataType: 'json',
						contentType: 'application/json; charset=utf-8', 
						crossDomain: true
					}).done(function(data){
						//console.log("Debug: Success in getting the data under DSSWYM");
						stringify = JSON.stringify(data);
						extract = JSON.parse(stringify);
						//console.log(extract); // Debug Check point
						stringify=null; 
							
							for (var r=0; r<10;r++){ 

								dsIdeaTitle = extract.result[r].title; // Refer do while block comments.
								dsIdeaTitle = convertXML2Char(dsIdeaTitle);
								author = extract.result[r].author.last_name + " " + extract.result[r].author.first_name;
								author = convertXML2Char(author);
								authorId = extract.result[r].author.login;
								community = extract.result[r].community_title;
								community = convertXML2Char(community);					
								thumbnail_id = extract.result[r]['first_media_id'];
								lastLink = "https://"+dsSwymUrl+"/#community:"+extract.result[r].community_id+"/ideation:"+extract.result[r].id;						
								
								 //console.log("Debug: For the value of r: "+r+" value of initialdsp and decoy are:"+historyCheck+" and "+decoy);  // Debug Check point 
												
								if(historyCheck == lastLink){  
									//console.log("Debug: found match at: "+r+ " There is no update yet"); // Debug Check point
									data = null;
									extract = null;
									return;
								}
								
								if((decoy == lastLink)&& (r!=9)){
									//console.log("Debug: decoy at action under DSSWYM Idea"); 				 								 		// Debug Check point
									chrome.storage.sync.set({'lastDsIdea': lastLink}, function () {
										//console.log("Debug: Last Idea notification for DSSWYM is updated to: "+lastLink); 		// Debug Check point
									});
									initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[r+1].community_id+"/ideation:"+extract.result[r+1].id;
									chrome.storage.sync.set({'decoyIdea': initialDecoyCounter}, function () {
										//console.log("Debug: Last Idea notification for DSSWYM is updated to: "+initialDecoyCounter);// Debug Check point
									});
									data = null;
									extract = null;
									return; 
								}
								
								dsIdeanotifier(dsIdeaTitle,lastLink,author,authorId,community,thumbnail_id);			// Call to notification function with parameters of information
								
								if(r==0){  // Core logic of this code is to store the latest update e.g. first avaialble Idea from JSON to be compared with next set of updates in JSON. Hence r==0 is evaluated.
									chrome.storage.sync.set({'lastDsPost': lastLink}, function () {
										//console.log("Debug: After Notification update to lastdspost "+lastLink);			// Debug Check point
									});
									initialDecoyCounter = "https://"+dsSwymUrl+"/#community:"+extract.result[r+1].community_id+"/ideation:"+extract.result[r+1].id;
									chrome.storage.sync.set({'decoyIdea': initialDecoyCounter}, function () {
										//console.log("Debug: After Notification update to decoy: "+initialDecoyCounter);	// Debug Check point
									});
								}
							} 
							data = null;
							extract = null;
						
					}).fail(function(xhr, status, error){
						console.log("DSIdea xhr log: "+xhr); 			// Message to user
						console.log("DSIdea Status log: "+status);		// Message to user
						console.log("DSIdea Error log: "+error);		// Message to user
						if(error == "Unauthorized" || error == "Authorization Required"){				// This log would tell how often token fails to make service calls or validity of token can be measured, reliability of token
							console.log("Authorization required, call to function authenticateDSSwym");	// Message to user
							authenticateDSSwym();
						}
						if(error == "Forbidden"){														// This log would fetch the fresh token in case of forbidden error // Need to investigate this type of error cause
							console.log("Call to SSO made from DSWYM Idea Function");					// Message to user
							getSSO();				
						}			
					});
				}
			}else if(timer == "errorHit"){
				clearInterval(delayBolt);
				return;
			}
		}, 100);	*/	
	}	
	
	
	//Based on user preference stored in local storage for enable/disable 3DSWYM notification, this function is called.
	//This function calls d3post & d3Iquestion function every 30 seconds.		
	setInterval(function(){
		chrome.storage.sync.get('d3SwymNotification', function (val) {
		//console.log("Debug: Do i've the permission to fetch 3DSWYM notification for User: "+ val.d3SwymNotification);  // Debug Check point
			if(val.d3SwymNotification =='yes'){
				if((d3SwymUrl != "") && (d3SwymUrl != undefined)){
					d3post();
					d3Iquestion();
				}else{
					getURLFromUser();	
				}
			}
		});
	//}, 15000);
	//}, 70000);	 
	}, 1000000);	 
		
	// set of functions to make web service calls to get bulk information and send individual piece of data to respective notificaiton function.
	// It has history tracker, which tracks the record of already displayed notificaiton upto 2 last displayed notification.
	function d3post(){
		var d3PostTitle, lastLink, author, authorId, community, extract, thumbnail_id; 
		var historyCheck, decoy, initialDecoyCounter; 
		var stringify;
		var timer;
		
		if(d3SwymUrl == undefined || d3SwymUrl== ""){
			console.log('User has to define 3DSWYM URL');
			getURLFromUser();			
		}
		
		chrome.storage.sync.get('last3dpost', function (val) {
			historyCheck = val.last3dpost;
			//console.log("Debug: Initializing the value from storage to last3dpost : "+historyCheck);
		});
		
		chrome.storage.sync.get('decoyost', function (val) {
			decoy = val.decoyost;
			//console.log("Debug: Intializing the value from storage to decoy for post update : "+decoy);
		});

		chrome.storage.sync.get('SSO3d', function (val) {
			sso3d = val.SSO3d;
			//console.log("Debug: Value got for 3DSWYM SSO: "+ sso3d);
		});		

		timer = 'start';
		var i = 0;
		do {
			//console.log("Debug: Economy loop exectuted in 3DPost");
			i += 1;
				$.ajax({
					url:'https://'+d3SwymUrl+'/api/filter/post',
					type: "GET",
					data:{"params":{"community_id":null,"limit":2}},		    
					headers: {'X-Requested-With': 'XMLHttpRequest','X-SWYM': sso3d },
					dataType: 'json',
					contentType: 'application/json; charset=utf-8', 
					crossDomain: true
					}).done(function(data){
						//console.log("Debug: Economy loop executed in 3DSWYM Post");
						stringify = JSON.stringify(data);
						extract = JSON.parse(stringify);
						//console.log(extract);
						stringify=null;
						
						if(historyCheck == undefined || historyCheck == ""){
						/*	//  Required information are title, author, user_id, community_name, thumbnail_id, postlink
								d3PostTitle = extract.result[0].title;
								console.log("d3PostTitle: "+d3PostTitle);
								author = extract.result[0]['user_id.last_name'] + " " + extract.result[0]['user_id.first_name'];
								console.log("author: "+author);
								authorId = extract.result[0]['user_id.hash_key'];
								console.log("authorId: "+authorId);
								community = extract.result[0].communityTitle;
								console.log("community: "+community); 
								thumbnail_id = extract.result[0]['this->getFirstMediaId'];	
								console.log("thumbnail_id : "+thumbnail_id);
								//lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[0].community_id+"/post:"+extract.result[0].community_post_id;
								lastLink = "https://"+d3SwymUrl+"/#post:"+extract.result[0].community_post_id;
								console.log("lastLink: "+lastLink); */

							d3PostTitle = extract.result[0].title;
							d3PostTitle = convertXML2Char(d3PostTitle);
							author = extract.result[0]['user_id.last_name'] + " " + extract.result[0]['user_id.first_name'];
							author = convertXML2Char(author);
							authorId = extract.result[0]['user_id.hash_key'];
							community = extract.result[0].communityTitle;
							community = convertXML2Char(community);
							thumbnail_id = extract.result[0]['this->getFirstMediaId'];
							//lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[0].community_id+"/post:"+extract.result[0].community_post_id; // This is adaptation of internal SWYM URL
							lastLink = "https://"+d3SwymUrl+"/#post:"+extract.result[0].community_post_id;
					
							//initialDecoyCounter =  "https://"+d3SwymUrl+"/#commun1ity:"+extract.result[1].community_id+"/post:"+extract.result[1].community_post_id;
							initialDecoyCounter =  "https://"+d3SwymUrl+"/#post:"+extract.result[1].community_post_id;
							
							chrome.storage.sync.set({'last3dpost': lastLink}, function () {
								//console.log("Debug: Last post notification is stored for 3DSWYM Post: "+ lastLink); 				// Debug Check point
							});	
							chrome.storage.sync.set({'decoyost': initialDecoyCounter}, function () {
								//console.log("Debug: Last but one Post notification for 3DSWYM is stored: "+initialDecoyCounter); // Debug Check point
							});
					
							swympostnotifier(d3PostTitle,lastLink,author,authorId,community,thumbnail_id); 
							data=null;
							extract=null;
							//console.log("Debug: Break command processed by POST function");
							timer = 'finished';	
							return;
							console.log("~~~~~~~~Break not honoured under post function, This should not happen~~~~~~~~");						
						}				
						lastLink = "https://"+d3SwymUrl+"/#post:"+extract.result[0].community_post_id;
				
						data  = null;
						extract = null;	
						timer = 'finished';							
					}).fail(function(xhr, status, error){
						timer = 'errorHit';
						console.log("3dSWYM POST XHR: "+xhr);
						console.log("3dSWYM POST Status: "+status);
						console.log("3dSWYM POST Error: "+error);
						if(error == "Unauthorized" || error == "Authorization Required"){
							//console.log("Debug: Authorization required, call to function authenticate3dSwym");			// Debug Check point
							authenticate3dSwym();
						}else {
							// Nothing here
						}
						if(error == "Forbidden"){
							//console.log("Debug: Call to SSO made from 3DSWYM Post Function");
							getSSO3d();
						}
					});						
		} while (i < 1);	
		i=null;						

		var delayBolt = setInterval(function () {
			if(timer == "finished"){
				clearInterval(delayBolt);
				
				if(historyCheck == lastLink || historyCheck == undefined){  
					//console.log("Debug: Found match at starting, there is no update yet in 3DSWYM Post"); // Debug Check point
					return; 
				}else {
					//console.log("Debug: Updates happened, started processing under 3DSWYM Post function"); // Debug check point
					$.ajax({
						url:'https://'+d3SwymUrl+'/api/filter/post',
						type: "GET",
						data:{"params":{"community_id":null,"limit":10}},		    
						headers: {'X-Requested-With': 'XMLHttpRequest','X-SWYM': sso3d },
						dataType: 'json',
						contentType: 'application/json; charset=utf-8', 
						crossDomain: true
					}).done(function(data){
						//console.log("Debug: Success in getting the data under 3DSWYM");
						stringify = JSON.stringify(data);
						extract = JSON.parse(stringify);
						//console.log(extract);
						stringify=null;		
						
							for (var p=0; p<10;p++){ 

								d3PostTitle = extract.result[p].title;
								d3PostTitle = convertXML2Char(d3PostTitle);
								author = extract.result[p]['user_id.last_name'] + " " + extract.result[p]['user_id.first_name'];
								author = convertXML2Char(author);
								authorId = extract.result[p]['user_id.hash_key'];
								community = extract.result[p].communityTitle;
								community = convertXML2Char(community);
								thumbnail_id = extract.result[p]['this->getFirstMediaId'];
								//lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[p].community_id+"/post:"+extract.result[p].community_post_id;						
								lastLink = "https://"+d3SwymUrl+"/#post:"+extract.result[p].community_post_id;						
								
								//console.log("Debug: For the value of r: "+p+" value of history and decoy are:"+historyCheck+" and "+decoy);  // Debug Check point \
								
								if(historyCheck == lastLink){  
									//console.log("Debug: found match at: "+p+"There is no update yet in 3DSWYM"); // Debug Check point
									data = null;
									extract = null;
									return; 
								}
								
								if((decoy == lastLink)&& (p!=9)){
									//console.log("Debug: decoy at action in 3DSWYM Post");	// Debug Check point
									
									chrome.storage.sync.set({'last3dpost': lastLink}, function () {
										//console.log("Debug: Last post notification for 3DSWYM is updated to : "+lastLink); // Debug Check point
									});
									//initialDecoyCounter =  "https://"+d3SwymUrl+"/#community:"+extract.result[p+1].community_id+"/post:"+extract.result[p+1].community_post_id;
									initialDecoyCounter =  "https://"+d3SwymUrl+"/#post:"+extract.result[p+1].community_post_id;
									chrome.storage.sync.set({'decoyost': initialDecoyCounter}, function () {
										//console.log("Debug: Last decoy Post notification for 3DSWYM is updated to : "+initialDecoyCounter);	// Debug Check point
									});
									data = null;
									extract=null;
									return; 
								}
								
								swympostnotifier(d3PostTitle,lastLink,author,authorId,community,thumbnail_id);  // Calls to Notification function.
								
								if(p==0){
									chrome.storage.sync.set({'last3dpost': lastLink}, function () {
										//console.log("Debug: After Notification update to last3dp from 3dswym: "+lastLink);			// Debug Check point
									});
									//initialDecoyCounter = "https://"+d3SwymUrl+"/#community:"+extract.result[p+1].community_id+"/post:"+extract.result[p+1].community_post_id;
									initialDecoyCounter = "https://"+d3SwymUrl+"/#post:"+extract.result[p+1].community_post_id;
									chrome.storage.sync.set({'decoyQuestion': initialDecoyCounter}, function () {
										//console.log("Debug: After Notification update to decoy from 3dswym: "+initialDecoyCounter);  // Debug Check point
									});
								}
							}
							data = null;
							extract =null;				
					}).fail(function(xhr, status, error){
						console.log("3dSWYM POST XHR: "+xhr);
						console.log("3dSWYM POST Status: "+status);
						console.log("3dSWYM POST Error: "+error);
						if(error == "Unauthorized" || error == "Authorization Required"){
							//console.log("Authorization required, call to function authenticate3dSwym");			// Debug Check point
							authenticate3dSwym();
						}else {
							// Nothing here
						}
						if(error == "Forbidden"){
								console.log("Call to SSO made from 3DSWYM Post Function");
							getSSO3d();
						}
					});					
				} 		
			}else if(timer == "errorHit"){
				clearInterval(delayBolt);
				return;
			}
		}, 100);
	}
				
	function d3Iquestion(){
		var title3DQuestion, lastLink, extract, author, authorId, community, thumbnail_id;  		
		var historyCheck, decoy, initialDecoyCounter; 
		var stringify;
		var timer;

		if(d3SwymUrl == undefined || d3SwymUrl== ""){
			console.log('User has to define 3DSWYM URL');  	//User message
			getURLFromUser();						
		}	
		
		chrome.storage.sync.get('decoyuestion', function (val) {
			decoy = val.decoyuestion;
			//console.log("Debug: Initializing the value from sync to decoy for decoy action : "+decoy);								// Debug Check point
		}); 
		
		chrome.storage.sync.get('last3dquestion', function (val) {
			historyCheck = val.last3dquestion;
			//console.log("Debug: Initializing the value from sync to historyCheck for 3DWYM Iquestion update : "+historyCheck);			// Debug Check point
		});		

		timer = 'start';
		var i = 0;
		do {
			//console.log("Debug: Economy loop exectuted in 3DQuestion");
			i += 1;
			$.ajax({				
				url:'https://'+d3SwymUrl+'/api/IquestionsQuestions/list',			
				type: "GET",
				data:{"params":{"community_id":null,"published":1,"limit":2}},
				headers: {'X-Requested-With': 'XMLHttpRequest','X-SWYM': sso3d },
				dataType: 'json',
				contentType: 'application/json; charset=utf-8',
				crossDomain: true
			}).done(function(data){
				stringify = JSON.stringify(data);
				extract = JSON.parse(stringify);
				//console.log(extract);   //Debug checkpoint
				stringify=null;
		/*	//  Required information are title, author, user_id, community_name, thumbnail_id, postlink
				title3DQuestion = extract.result[0].title;
				console.log("title3DQuestion: "+title3DQuestion);
				author = extract.result[0].nickname;
				console.log("author: "+author);
				authorId = extract.result[0].hash_key;
				console.log("authorId: "+authorId);
				community = extract.result[0].community_title;
				console.log("community: "+community); 
				//thumbnail_id = extract.result[0].first_media_id;
				//console.log("thumbnail_id : "+thumbnail_id);
				qna_id = extract.result[0].qna_id;
				console.log("qna_id : "+qna_id);
				lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[0].community_id+"/iquestions:"+extract.result[0].qna_id;
				console.log("lastLink: "+lastLink); 	*/					
				
				if(historyCheck == undefined || historyCheck == ""){

					title3DQuestion = extract.result[0].title;
					title3DQuestion = convertXML2Char(title3DQuestion);
					author = extract.result[0].nickname;
					author = convertXML2Char(author);
					authorId = extract.result[0].hash_key;
					community = extract.result[0].community_title;
					community = convertXML2Char(community);
					//qna_id = extract.result[0].qna_id;
					//thumbnail_id = extract.result[0].first_media_id;	//This information is not available from JSON data, SR raised to provide this id.
 				
					lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[0].community_id+"/iquestions:"+extract.result[0].qna_id;
					
					initialDecoyCounter = "https://"+d3SwymUrl+"/#community:"+extract.result[1].community_id+"/iquestions:"+extract.result[1].qna_id;
					
					chrome.storage.sync.set({'last3dquestion': lastLink}, function () {
						//console.log("Last IQuestion notification is stored for 3DSWYM "+lastLink);						// Debug Check point
					});
					chrome.storage.sync.set({'decoyuestion': initialDecoyCounter}, function () {
						//console.log("Last but one IQuestion notification for 3DSWYM is stored"+initialDecoyCounter);		// Debug Check point
					});			
					
					if(title3DQuestion !=""){
						swymIQuestionnotifier(title3DQuestion,lastLink,author,authorId,community); 			// Call to notification function			
						data = null;
						extract = null;
					}
					//console.log("Debug: Break command processed by Question Function");
					timer = 'finished';
					return;
					console.log("~~~~~~~~~Break not honoured under Question Function. This should not happen~~~~~~~~");					
				}				
				lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[0].community_id+"/iquestions:"+extract.result[0].qna_id;
				data  = null;
				extract = null;					
				timer = 'finished';			
			}).fail(function(xhr, status, error){
				timer = 'errorHit';
				console.log("3DSWYM Question XHR: "+xhr);
				console.log("3DSWYM Question Status: "+status);
				console.log("3DSWYM Question Error: "+error);
				if(error =="Unauthorized"){
					console.log("Authorization Required, a message from 3DSWYM IQuestion Function"); // User message, d3post function handles this error message.
				}
			}); 	
		} while (i < 1);	
		i=null;			

		var delayBolt = setInterval(function () {
			if(timer == "finished"){
				clearInterval(delayBolt);
						
				if(historyCheck == lastLink || historyCheck == undefined){  
					//console.log("Debug: Found match at starting, there is no update yet in 3DSWYM IQuestion"); // Debug Check point
					return; 
				}else{	
					//console.log("Debug: Updates happened, started processing under 3DSWYM IQuestion function"); // Debug check point	
					
					$.ajax({
						url:'https://'+d3SwymUrl+'/api/IquestionsQuestions/list',			
						type: "GET",
						data:{"params":{"community_id":null,"published":1,"limit":10}},
						headers: {'X-Requested-With': 'XMLHttpRequest','X-SWYM': sso3d },
						dataType: 'json',
						contentType: 'application/json; charset=utf-8',
						crossDomain: true
					}).done(function(data){
						stringify = JSON.stringify(data);
						extract = JSON.parse(stringify);
						//console.log(extract);   //Debug checkpoint
						stringify=null;	
			
							for (var i=0; i<10;i++){ 								
								title3DQuestion = extract.result[i].title;
								title3DQuestion = convertXML2Char(title3DQuestion);
								author = extract.result[i].nickname;
								author = convertXML2Char(author);
								authorId = extract.result[i].hash_key;
								community = extract.result[i].community_title;
								community = convertXML2Char(community);
								//qna_id = extract.result[i].qna_id;
								//thumbnail_id = extract.result[i].first_media_id;
								lastLink = "https://"+d3SwymUrl+"/#community:"+extract.result[i].community_id+"/iquestions:"+extract.result[i].qna_id;
						
								//console.log("Debug: For the value of i: "+i+" value of initialdsq and decoy are:"+historyCheck+" and "+decoy);
						
								if(historyCheck == lastLink){  
									//console.log("Debug: Found match at: "+i+" There is no update yet for 3DIQuestion"); 			// Debug check point
									data=null;
									extract = null;
									return; 
								}
								
								if(decoy == lastLink){	
									//console.log("Debug: decoy at action in 3DSWYM IQuestion"); 										   // Debug check point
									chrome.storage.sync.set({'last3dquestion': lastLink}, function () {
										//console.log("Debug: Last IQuestion notification for 3DSWYM updated to : "+lastLink);			// Debug check point
									});
									initialDecoyCounter = "https://"+d3SwymUrl+"/#community:"+extract.result[i+1].community_id+"/iquestions:"+extract.result[i+1].qna_id;
									chrome.storage.sync.set({'decoyuestion': initialDecoyCounter}, function () {
										//console.log("Debug: Last decoy IQuestion notification for 3DSWYM updated to: "+initialDecoyCounter);	// Debug check point
									});
									data = null;
									extract = null;
									return; 
								}
								
								if(title3DQuestion != ""){
									swymIQuestionnotifier(title3DQuestion,lastLink,author,authorId,community);
								
									if(i==0){
										chrome.storage.sync.set({'last3dquestion': lastLink}, function () {
											//console.log("Debug: After Notification update to last3dq "+lastLink);								// Debug check point
										});
										initialDecoyCounter = "https://"+d3SwymUrl+"/#community:"+extract.result[i+1].community_id+"/iquestions:"+extract.result[i+1].qna_id;
										chrome.storage.sync.set({'decoyQuestion': initialDecoyCounter}, function () {
											//console.log("Debug: After Notification update to decoy "+initialDecoyCounter);						// Debug check point
										});						
									}
								}
							} 					
							data=null;
							extract = null;	 
					}).fail(function(xhr, status, error){
						console.log("3DSWYM Question XHR: "+xhr);
						console.log("3DSWYM Question Status: "+status);
						console.log("3DSWYM Question Error: "+error);
						if(error =="Unauthorized"){
							console.log("Authorization Required, a message from 3DSWYM IQuestion Function"); // User message, d3post function handles this error message.
						}
					}); 
				}
			}else if(timer == "errorHit"){
				clearInterval(delayBolt);
				return;
			} 	
		}, 100);
	}
	
	// Function to display individual notification for DSSWYM Portal
	function dspostnotifier(dsPostTitle,lastLink,author,authorId,community,thumbnail_id){
	var dsSwymNotificationClose, notificaitonTypeDSSwym; 	// This variable declared to be used within nested functions	

		chrome.storage.sync.get('dsSwymNotificationClose', function (val) {
			//console.log("Debug: Closing in:"+ val.dsSwymNotificationClose); 								// Debug Check point
			dsSwymNotificationClose = val.dsSwymNotificationClose;
			if(val.dsSwymNotificationClose == 'manual')												// This block produces manually closing notification type
			{	
				chrome.storage.sync.get('notificaitonTypeDSSwym', function (val){
					notificaitonTypeDSSwym = val.notificaitonTypeDSSwym;
					if(notificaitonTypeDSSwym =="basic"){
					chrome.notifications.create(lastLink,											// Here is the beauty, post or question url is stored in lastlink variable, that variable is used as id to each notification. refer onclick listener function at last.
						{type: "basic",																// Whether BASIC or IMAGE type of notification 
						title: 'Post by: '+author,
						message: dsPostTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
						requireInteraction: true													// This means, notification won't close untill User closes it manually
						});			
					}else if (notificaitonTypeDSSwym =="image" && thumbnail_id != null ){			// When there is no image, better to switch to basic notification rather than displaying a black image window
					chrome.notifications.create(lastLink,
						{type: "image",
						title: 'Post by: '+author,
						message: dsPostTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
						imageUrl: 'https://'+dsSwymUrl+'/api/media/stream/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
						requireInteraction: true
						});	
					}else {
					chrome.notifications.create(lastLink,
						{type: "basic",
						title: 'Post by: '+author,
						message: dsPostTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
						requireInteraction: true
						});							
					}					
				});
			} 
			else {																					// This block produces duration based notification close type
				chrome.storage.sync.get('notificaitonTypeDSSwym', function (val){
					notificaitonTypeDSSwym = val.notificaitonTypeDSSwym;
					if(notificaitonTypeDSSwym =="basic"){				
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Post by: '+author,
							message: dsPostTitle,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
							eventTime: Number(dsSwymNotificationClose)
						});				
					}else if (notificaitonTypeDSSwym =="image" && thumbnail_id != null ){
					chrome.notifications.create(lastLink,
						{type: "image",
						title: 'Post by: '+author,
						message: dsPostTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
						imageUrl: 'https://'+dsSwymUrl+'/api/media/stream/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
						eventTime: Number(dsSwymNotificationClose)
						});	
					}else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Post by: '+author,
							message: dsPostTitle,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
							eventTime: Number(dsSwymNotificationClose)
						});							
					}
				});
			}
		}); 
	}
	
	function dsIQuestionnotifier(titleDsQuestion,lastLink,author,authorId,community,thumbnail_id){
	var dsSwymNotificationClose, notificaitonTypeDSSwym;
	
		chrome.storage.sync.get('dsSwymNotificationClose', function (val) {
			dsSwymNotificationClose = val.dsSwymNotificationClose;
			//console.log("Debug: Closing in:"+ val.dsSwymNotificationClose);
			if(val.dsSwymNotificationClose == 'manual'){	
				chrome.storage.sync.get('notificaitonTypeDSSwym', function (val){
					notificaitonTypeDSSwym = val.notificaitonTypeDSSwym;
					if(notificaitonTypeDSSwym =="basic"){
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: titleDsQuestion,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
							requireInteraction: true	
						});
					}else if(notificaitonTypeDSSwym =="image" && thumbnail_id != null ){
						chrome.notifications.create(lastLink,
							{type: "image",
							title: 'Question by: '+author,
							message: titleDsQuestion,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
							imageUrl: 'https://'+dsSwymUrl+'/api/media/stream/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
							requireInteraction: true
						});						
					} else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: titleDsQuestion,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
							requireInteraction: true	
						});						
					}				
				});
			}else{
				chrome.storage.sync.get('notificaitonTypeDSSwym', function (val){
					notificaitonTypeDSSwym = val.notificaitonTypeDSSwym;
					if(notificaitonTypeDSSwym =="basic"){
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: titleDsQuestion,
							contextMessage : community,
							iconUrl: "IQuestion.png",
							eventTime: Number(dsSwymNotificationClose)
						});		
					}else if(notificaitonTypeDSSwym =="image" && thumbnail_id != null ){
						chrome.notifications.create(lastLink,
							{type: "image",
							title: 'Question by: '+author,
							message: titleDsQuestion,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
							imageUrl: 'https://'+dsSwymUrl+'/api/media/stream/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
							eventTime: Number(dsSwymNotificationClose)
						});						
					}else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: titleDsQuestion,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
							eventTime: Number(dsSwymNotificationClose)
						});								
					}
				});
			}	
		}); 
	}	

	// Function to display individual notification for DSSWYM Portal
	function dsIdeanotifier(dsIdeaTitle,lastLink,author,authorId,community,thumbnail_id){
	var dsSwymNotificationClose, notificaitonTypeDSSwym; 	// This variable declared to be used within nested functions	

		chrome.storage.sync.get('dsSwymNotificationClose', function (val) {
			//console.log("Debug: Closing in:"+ val.dsSwymNotificationClose); 								// Debug Check point
			dsSwymNotificationClose = val.dsSwymNotificationClose;
			if(val.dsSwymNotificationClose == 'manual')												// This block produces manually closing notification type
			{	
				chrome.storage.sync.get('notificaitonTypeDSSwym', function (val){
					notificaitonTypeDSSwym = val.notificaitonTypeDSSwym;
					if(notificaitonTypeDSSwym =="basic"){
					chrome.notifications.create(lastLink,											// Here is the beauty, Idea or question url is stored in lastlink variable, that variable is used as id to each notification. refer onclick listener function at last.
						{type: "basic",																// Whether BASIC or IMAGE type of notification 
						title: 'Idea by: '+author,
						message: dsIdeaTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
						requireInteraction: true													// This means, notification won't close untill User closes it manually
						});			
					}else if (notificaitonTypeDSSwym =="image" && thumbnail_id != null ){			// When there is no image, better to switch to basic notification rather than displaying a black image window
					chrome.notifications.create(lastLink,
						{type: "image",
						title: 'Idea by: '+author,
						message: dsIdeaTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
						imageUrl: 'https://'+dsSwymUrl+'/api/media/stream/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
						requireInteraction: true
						});	
					}else {
					chrome.notifications.create(lastLink,
						{type: "basic",
						title: 'Idea by: '+author,
						message: dsIdeaTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
						requireInteraction: true
						});							
					}					
				});
			} 
			else {																					// This block produces duration based notification close type
				chrome.storage.sync.get('notificaitonTypeDSSwym', function (val){
					notificaitonTypeDSSwym = val.notificaitonTypeDSSwym;
					if(notificaitonTypeDSSwym =="basic"){				
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Idea by: '+author,
							message: dsIdeaTitle,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
							eventTime: Number(dsSwymNotificationClose)
						});				
					}else if (notificaitonTypeDSSwym =="image" && thumbnail_id != null ){
					chrome.notifications.create(lastLink,
						{type: "image",
						title: 'Idea by: '+author,
						message: dsIdeaTitle,
						contextMessage : community,
						iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',
						imageUrl: 'https://'+dsSwymUrl+'/api/media/stream/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
						eventTime: Number(dsSwymNotificationClose)
						});	
					}else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Idea by: '+author,
							message: dsIdeaTitle,
							contextMessage : community,
							iconUrl: 'https://'+dsSwymUrl+'/api/user/getpicture/login/'+authorId+'/format/normal',	
							eventTime: Number(dsSwymNotificationClose)
						});							
					}
				});
			}
		}); 
	}
	
	// Function to display individual notification for 3DSWYM Portal
	function swympostnotifier(d3PostTitle,lastLink,author,authorId,community,thumbnail_id){
	var d3SwymNotificationClose, notificaitonType3dswym;
		
		chrome.storage.sync.get('d3SwymNotificationClose', function (val) {
			d3SwymNotificationClose = val.d3SwymNotificationClose;	
			//console.log("Debug: closing in: "+d3SwymNotificationClose);
			if(d3SwymNotificationClose == 'manual'){
				chrome.storage.sync.get('notificaitonType3dswym', function (val){	
					notificaitonType3dswym = val.notificaitonType3dswym;			
					if (notificaitonType3dswym == "basic"){
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Post by: '+author,
							message: d3PostTitle,
							contextMessage : community,
							iconUrl:'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',	 	
							requireInteraction: true
						});		
					}else if (notificaitonType3dswym =="image" && thumbnail_id != null ){
						chrome.notifications.create(lastLink,
							{type: "image",
							title: 'Post by: '+author,
							message: d3PostTitle,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							imageUrl: 'https://'+d3SwymUrl+'/api/media/streammedia/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
							requireInteraction: true
							});	
					}else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Post by: '+author,
							message: d3PostTitle,
							contextMessage : community,
							iconUrl:'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',	 	
							requireInteraction: true
						});								
					}
				});
			}else{
				chrome.storage.sync.get('notificaitonType3dswym', function (val){	
					notificaitonType3dswym = val.notificaitonType3dswym;			
					if (notificaitonType3dswym == "basic"){				
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Post by: '+author,
							message: d3PostTitle,
							contextMessage : community,
							iconUrl:'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',	 	
							eventTime: Number(d3SwymNotificationClose)
						});						
					}else if (notificaitonType3dswym =="image" && thumbnail_id != null ){
						chrome.notifications.create(lastLink,
							{type: "image",
							title: 'Post by: '+author,
							message: d3PostTitle,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							imageUrl: 'https://'+d3SwymUrl+'/api/media/streammedia/id/'+thumbnail_id+'/key/l_thumb/type/thumb',
							eventTime: Number(d3SwymNotificationClose)
							});	
					}else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Post by: '+author,
							mesage: d3PostTitle,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							eventTime: Number(d3SwymNotificationClose)
						});								
					}
				});
			}
		}); 
	}
	
	function swymIQuestionnotifier(title3DQuestion,lastLink,author,authorId,community){
	var d3SwymNotificationClose, notificaitonType3dswym;

		chrome.storage.sync.get('d3SwymNotificationClose', function (val) {
			var d3SwymNotificationClose = val.d3SwymNotificationClose;	
			//console.log("Debug: Closing in: "+d3SwymNotificationClose );
			if(d3SwymNotificationClose == 'manual'){
				chrome.storage.sync.get('notificaitonType3dswym', function (val){	
					notificaitonType3dswym = val.notificaitonType3dswym;			
					if (notificaitonType3dswym == "basic"){
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: title3DQuestion,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							requireInteraction: true
						}); 		
					}
					else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: title3DQuestion,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							requireInteraction: true
						}); 							
					}
				});
			}else{
				chrome.storage.sync.get('notificaitonType3dswym', function (val){	
					notificaitonType3dswym = val.notificaitonType3dswym;			
					if (notificaitonType3dswym == "basic"){				
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: title3DQuestion,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							eventTime: Number(d3SwymNotificationClose)
						}); 					
					}else{
						chrome.notifications.create(lastLink,
							{type: "basic",
							title: 'Question by: '+author,
							message: title3DQuestion,
							contextMessage : community,
							iconUrl: 'https://'+d3SwymUrl+'/api/user/getpicture/hash_key/'+authorId+'/format/medium',
							eventTime: Number(d3SwymNotificationClose)
						}); 							
					}
				});
			}
		}); 
	}		

	
	// Function to handle notification on click, to open the respective post or Iquestion for the user.
	chrome.notifications.onClicked.addListener(function (lastLink){				//This call back function will return the id of the notification, which as mentioned during creation.
		if((lastLink=="dsswym")||(lastLink=="3dswym")){
			//console.log("Debug: This click should not be executed and hence quit the listener function");
			return;
		}
		chrome.windows.getAll({populate : false}, function (window_list) {		
			if (window_list == 0){												// If chrome browser is not open, it will open a fresh browser and navigates to the URL
				chrome.windows.create({url: lastLink, type: "normal"}); 		// We got the url and when user clicks, it will navigate to respective URL
			}else {
				chrome.tabs.create({url: lastLink,}, function(tab){				// If chrome browser is open, it will open a new tab and navigates to the URL
					//nothing here
				});
			}
		});	
		chrome.notifications.clear(lastLink);									// Finally, it will close the notification, no matter whether notificaiton type is manual or time dependent. Again the beauty is it closes based on the ID			
	});
	
	chrome.notifications.onButtonClicked.addListener(function(notificationId, index){
		if (index==0 && notificationId =="dsswym"){
			chrome.windows.getAll({populate : false}, function (window_list) {
				if (window_list == 0){
					chrome.windows.create({url: 'https://'+dsSwymUrl+'/api/passport/login?redirect=https://'+dsSwymUrl+'/', type: "normal"});
				}else {
					chrome.tabs.create({url: 'https://'+dsSwymUrl+'/api/passport/login?redirect=https://'+dsSwymUrl+'/'}, function(tab){
						//nothing here
					});
				}				
			});			
			chrome.notifications.clear("dsswym");
		}else if (index==1 && notificationId =="dsswym"){
			chrome.storage.sync.set({'dsSwymNotification': "No"}, function () {
				console.log("DS SWYM Notification is disabled, on users request");		// Debug Check point		
			});	
			chrome.notifications.clear("dsswym");
		}
		
		if (index==0 && notificationId =="3dswym"){
			chrome.windows.getAll({populate : false}, function (window_list) {
				if (window_list == 0){
					chrome.windows.create({url: 'https://'+d3SwymUrl+'/api/passport/login?redirect=https://'+d3SwymUrl+'/', type: "normal"});
				}else {
					chrome.tabs.create({url: 'https://'+d3SwymUrl+'/api/passport/login?redirect=https://'+d3SwymUrl+'/'}, function(tab){
						//nothing here
					});
				}
			});	
			chrome.notifications.clear("3dswym");
		}else if (index==1 && notificationId =="3dswym"){
			chrome.storage.sync.set({'d3SwymNotification': "No"}, function () {
				console.log("3D SWYM Notification is disabled, on users request"); // Debug Check point				
			});			
			chrome.notifications.clear("3dswym");
		}		
	});

	
	// Refer WEBRequest API of chrome
	// This function is primarily to detect 3DPassport URL and make request to for authentication during 401 error.
	chrome.webRequest.onBeforeRedirect.addListener(function(details){
		var processor, status, redirectUrl;
		//console.log("Debug: Redirect status code: "+details.statusCode);	// Mostly will be 302
		//console.log("Debug: Redirect URL: "+details.redirectUrl);		 	// Redirect to Passport server.
		//console.log("Debug: URL "+details.url);								// 3DSWYM url
		//console.log("Debug: Headers"+details.responseHeaders);				// Response headers to fetch the redirect location	
		
		//If DSSWYM login api(/api/passport/login) triggers the redirect, which is to determine the PASSPORT server address.
		//dsSwymPassportUrl variable assigned with passport address
		if("https://"+dsSwymUrl+"/api/passport/login" == details.url){
			var urlDSP = details.responseHeaders.filter(function (Location) { 
				return Location.name == "Location"; 
			});			
			urlDSP = urlDSP[0].value;
			if(urlDSP.indexOf('.com/') > -1){	
				dsSwymPassportUrl=urlDSP.substring(0,urlDSP.indexOf(".com/")+5);	
				//console.log("Debug: DSSWYM 3DPassport url assigned is: "+dsSwymPassportUrl);   // Debug Check point
			}
			urlDSP=null;
		}	
		
		//If 3DSWYM login api (/api/passport/login) triggers the redirect, which is to determine the PASSPORT server address.
		//d3SwymPassportUrl variable assigned with passport address
		if("https://"+d3SwymUrl+"/api/passport/login" == details.url){
			var url3DP = details.responseHeaders.filter(function (Location) { 
				return Location.name == "Location"; 
			});			
			url3DP = url3DP[0].value;
			if(url3DP.indexOf('.com/') > -1){
				d3SwymPassportUrl=url3DP.substring(0,url3DP.indexOf(".com/")+4);
				//console.log("Debug: 3DSWYM Passport url assigned is: "+d3SwymPassportUrl);   // Debug Check point	
			}
			url3DP = null;
		}		
		
	}, {'urls': ['<all_urls>'],'types': ['xmlhttprequest']}, ['responseHeaders']);
	
	
